package com.podpoint.pptdeviceinfo;

import javax.annotation.Nullable;

import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;

import android.os.Build;
import android.provider.Settings.Secure;
import android.content.pm.PackageManager;
import android.content.pm.PackageInfo;

import java.util.HashMap;
import java.util.Map;

public class PPTDeviceInfoManager extends ReactContextBaseJavaModule {
    /**
     * The name of the react module.
     */
    public static final String REACT_CLASS = "PPTDeviceInfo";

    /**
     * The application context.
     */
    ReactApplicationContext reactContext;

    /**
     * Creates a new device info manager class.
     *
     * @param reactContext
     */
    public PPTDeviceInfoManager(ReactApplicationContext reactContext) {
        super(reactContext);

        this.reactContext = reactContext;
    }

    /**
     * Returns the name of the react module.
     *
     * @return String
     */
    @Override
    public String getName() {
        return REACT_CLASS;
    }

    /**
     * Returns the constants that this package sets.
     *
     * @return Map
     */
    @Override
    public @Nullable
    Map<String, Object> getConstants() {
        HashMap<String, Object> constants = new HashMap<String, Object>();

        PackageManager packageManager = this.reactContext.getPackageManager();
        String packageName = this.reactContext.getPackageName();

        constants.put("appVersion", "not available");
        constants.put("buildNumber", 0);

        try {
            PackageInfo info = packageManager.getPackageInfo(packageName, 0);
            constants.put("appVersion", info.versionName);
            constants.put("buildNumber", info.versionCode);
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }

        constants.put("systemName", "Android");
        constants.put("systemVersion", Build.VERSION.RELEASE);
        constants.put("model", Build.MODEL);
        constants.put("deviceToken", Build.BOARD);
        constants.put("hardwareId", Secure.getString(this.reactContext.getContentResolver(), Secure.ANDROID_ID));
        constants.put("systemManufacturer", Build.MANUFACTURER);
        constants.put("bundleId", packageName);

        return constants;
    }
}
