#import "PPTDeviceInfoProvider.h"

static NSString *deviceToken = @"";
static NSString *deviceTokenError = @"USER_NOTIFICATIONS_NOT_REGISTERED";

@implementation PPTDeviceInfoProvider

/**
 * Sets the device token so that it is available for use elsewhere in the application.
 *
 * @return void
 */
+ (void)provideDeviceToken:(NSString *)token
{
    deviceToken = token;
}

/**
 * Sets the device token error message so that it is available for use elsewhere in the application.
 *
 * @return void
 */
+ (void)provideDeviceTokenError:(NSString *)tokenError
{
    deviceTokenError = tokenError;
}

/**
 * Getter for the device token. Can transform the token into a plain string or return it in it's standard state
 * wrapped in <> tags with spaces.
 *
 * @return NSString
 */
+ (NSString *)getDeviceToken:(BOOL)transformToken
{
    if (transformToken) {
        return [[[deviceToken
                  stringByReplacingOccurrencesOfString: @"<" withString: @""]
                 stringByReplacingOccurrencesOfString: @">" withString: @""]
                stringByReplacingOccurrencesOfString: @" " withString: @""];
    }
    
    return deviceToken;
}

/**
 * Getter for the device token error message.
 *
 * @return NSString
 */
+ (NSString *)getDeviceTokenError
{
    return deviceTokenError;
}

/**
 * Whether or not a device token is available.
 *
 * @return NSString
 */
+ (BOOL)deviceTokenAvailable
{
    if ([deviceToken length] == 0) {
        return NO;
    }
    
    return YES;
}

@end
